@@ -2,27 +2,16 @@
 # encoding: utf-8
 
 # (c) 2016, Jiri Tyr <jiri.tyr@gmail.com>
-#
-# This file is part of Ansible
-#
-# Ansible is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# Ansible is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
+# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
 
-ANSIBLE_METADATA = {'metadata_version': '1.0',
+from __future__ import absolute_import, division, print_function
+__metaclass__ = type
+
+
+ANSIBLE_METADATA = {'metadata_version': '1.1',
                     'status': ['preview'],
                     'supported_by': 'community'}
 
-
 DOCUMENTATION = '''
 ---
 module: jenkins_plugin
@@ -63,6 +52,8 @@ options:
     description:
       - Option used to allow the user to overwrite any of the other options. To
         remove an option, set the value of the option to C(null).
+      - Changed in 2.5.0, 2.4.1, 2.3.3 to raise an error if C(url_password) is specified in params.
+        Use the actual C(url_password) argument instead.
   state:
     required: false
     choices: [absent, present, pinned, unpinned, enabled, disabled, latest]
@@ -177,20 +168,18 @@ EXAMPLES = '''
     state: absent
 
 #
-# Example of how to use the params
-#
-# Define a variable and specify all default parameters you want to use across
-# all jenkins_plugin calls:
+# Example of how to authenticate
 #
 # my_jenkins_params:
 #   url_username: admin
-#   url_password: p4ssw0rd
-#   url: http://localhost:8888
 #
 - name: Install plugin
   jenkins_plugin:
     name: build-pipeline-plugin
     params: "{{ my_jenkins_params }}"
+    url_password: p4ssw0rd
+    url: http://localhost:8888
+# Note that url_password **can not** be placed in params as params could end up in a log file
 
 #
 # Example of a Play which handles Jenkins restarts during the state changes
@@ -296,6 +285,7 @@ state:
 
 from ansible.module_utils.basic import AnsibleModule
 from ansible.module_utils.pycompat24 import get_exception
+from ansible.module_utils.six.moves.urllib.parse import urlencode
 from ansible.module_utils.urls import fetch_url, url_argument_spec
 from ansible.module_utils._text import to_native
 import base64
@@ -304,7 +294,6 @@ import json
 import os
 import tempfile
 import time
-import urllib
 
 
 class JenkinsPlugin(object):
@@ -330,7 +319,12 @@ class JenkinsPlugin(object):
         csrf_data = self._get_json_data(
             "%s/%s" % (self.url, "api/json"), 'CSRF')
 
-        return csrf_data["useCrumbs"]
+        if 'useCrumbs' not in csrf_data:
+            self.module.fail_json(
+                msg="Required fields not found in the Crumbs response.",
+                details=csrf_data)
+
+        return csrf_data['useCrumbs']
 
     def _get_json_data(self, url, what, **kwargs):
         # Get the JSON data
@@ -338,12 +332,12 @@ class JenkinsPlugin(object):
 
         # Parse the JSON data
         try:
-            json_data = json.load(r)
+            json_data = json.loads(to_native(r.read()))
         except Exception:
             e = get_exception()
             self.module.fail_json(
                 msg="Cannot parse %s JSON data." % what,
-                details=e.message)
+                details=to_native(e))
 
         return json_data
 
@@ -366,7 +360,7 @@ class JenkinsPlugin(object):
                 self.module.fail_json(msg=msg_status, details=info['msg'])
         except Exception:
             e = get_exception()
-            self.module.fail_json(msg=msg_exception, details=e.message)
+            self.module.fail_json(msg=msg_exception, details=to_native(e))
 
         return response
 
@@ -435,7 +429,7 @@ class JenkinsPlugin(object):
                     'script': install_script
                 }
                 script_data.update(self.crumb)
-                data = urllib.urlencode(script_data)
+                data = urlencode(script_data)
 
                 # Send the installation request
                 r = self._get_url_data(
@@ -723,7 +717,7 @@ class JenkinsPlugin(object):
     def _pm_query(self, action, msg):
         url = "%s/pluginManager/plugin/%s/%s" % (
             self.params['url'], self.params['name'], action)
-        data = urllib.urlencode(self.crumb)
+        data = urlencode(self.crumb)
 
         # Send the request
         self._get_url_data(
@@ -770,6 +764,11 @@ def main():
 
     # Update module parameters by user's parameters if defined
     if 'params' in module.params and isinstance(module.params['params'], dict):
+        if 'url_password' in module.params['params']:
+            # The params argument should be removed eventually.  Until then, raise an error if
+            # url_password is specified there as it can lead to the password being logged
+            module.fail_json(msg='Do not specify url_password in params as it may get logged')
+
         module.params.update(module.params['params'])
         # Remove the params
         module.params.pop('params', None)
